# Copyright 2004 Tom Rothamel <tom-renpy@onegeek.org>
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import cPickle as pickle

from character import Character
import fragment

class EndGameException:
    pass

class FragJumpException:
    def __init__(self, target):
        self.target = target

class Executive(object):
    """
    @ivar env: The environment that python code executes in.
    """

    def __init__(self):

        self.env = {
            'state' : [],
            'prompt' : self.prompt,
            'save' : self.save,
            }

        self.scene = [ ]

        # Perhaps do stuff here to export things into the game
        # namespace.


    def eval(self, cmd):
        return eval(cmd, self.env)

    def run(self, frag):

        while frag:

            # Checkpoint the current fragment, so we can save it if
            # needed.
            self.current_frag = frag.name

            try:
                self.ui_pre_frag()
                frag = frag.run(self)
                self.ui_post_frag()
                
            except FragJumpException, e:
                frag = fragment.lookup(e.target)

    def do_python(self, cmd):
        exec cmd in self.env

    def save(self, *vars):
        self.env['state'].extend(vars)

    def prompt(self, label, limit, default=""):
        return self.ui_prompt(label, limit, default)
        
    def do_dialog(self, who, what):

        if not who:
            self.ui_dialog_1(what % self.env)
            return

        # TODO: Error handling.

        who = self.eval(who)

        if not isinstance(who, str):
            who = who.name

        self.ui_dialog_2(who, what % self.env)

    def do_menu(self, label, choices):
        if label:
            label = label % self.env

        choices = [ i % self.env for i in choices ]

        return self.ui_menu(label, choices)
            

    def do_character(self, name):
        self.env[name] = Character(name)
        self.env['state'].append(name)

    def do_scene(self):
        self.scene = [ ]
        self.ui_reset_scene()

    def do_hide(self, thing):

        # return true if a is a prefix of b.
        def prefix(a, b):
            if len(b) < len(a):
                return False

            for av, bv in zip(a, b):
                if av != bv:
                    return False

            return True

        self.scene = [ i for i in self.scene if not prefix(thing, i) ]

    def do_show(self, thing):
        self.do_hide((thing[0],))
        self.scene.append(thing)
        
    def do_save(self, fn):
        env = [ (key, self.env[key]) for key in self.env['state'] ]

        print repr(env)
        state = self.current_frag, self.scene, env

        f = file(fn, "w")
        pickle.dump(state, f)
        f.close()

    def do_load(self, fn):
        f = file(fn)
        state = pickle.load(f)
        f.close()

        frag, self.scene, env = state
        
        for k, v in env:
            self.env[k] = v

        raise FragJumpException(frag)
            
