# Copyright 2004 Tom Rothamel <tom-renpy@onegeek.org>
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import display
import executive
import image
import layout
from params import config
import widget

class GUI(executive.Executive):

    def __init__(self):
        super(GUI, self).__init__()
        
        display.init()
        widget.init()
        self.images = {}
        self.escape_mode = False

    def interact(self, widget):
        display.show(widget)
        display.interact(self)
        display.hide(widget)

    def ui_pre_frag(self):
        display.clear()
        for t in self.scene:
            display.show(self.images[t])
        
    def ui_post_frag(self):
        pass

    def ui_prompt(self, msg, limit, default=""):
        prompt = widget.PromptBox(msg, limit, default)
        anchor = layout.Anchor(prompt, yo=config.display_yo)

        self.interact(anchor)

        return prompt.content
        
    
    def ui_dialog_1(self, what):
        msg = widget.MessageBox(what)
        anchor = layout.Anchor(msg, yo=config.display_yo)

        self.interact(anchor)
        
    def ui_dialog_2(self, who, what):
        label = widget.LabelBox(who)
        msg = widget.MessageBox(what)
        vbox = layout.VBox(label, msg)
        anchor = layout.Anchor(vbox, yo=config.display_yo)

        self.interact(anchor)

    def ui_menu(self, label, choices):
        menu = widget.MenuBox(label, choices)
        anchor = layout.Anchor(menu, yo=config.display_yo)

        self.interact(anchor)

        return menu.choice

    def ui_image(self, t, img):

        if isinstance(img, image.image):
            img = layout.Anchor(img)

        self.images[t] = img
            
    def ui_reset_scene(self):
        display.clear()
        display.reset_clock()

    def ui_center_menu(self, label, *choices):
        menu = widget.MenuBox(label, choices, width=config.center_menu_width)
        anchor = layout.Anchor(menu, yo=config.display_yo, valign="center")

        self.interact(anchor)

        return menu.choice
        
    def ui_escape(self):
        if self.escape_mode:
            return

        self.escape_mode = True

        while True:

            cmd = self.ui_center_menu("Command Menu\n",
                                      "Return to Game",
                                      "Save Game",
                                      "Load Game",
                                      "Quit Game")

            if cmd == 0:
                self.escape_mode = False
                return

            if cmd == 1:
                self.do_save("game.save")

            if cmd == 2:
                try:
                    self.do_load("game.save")
                except executive.FragJumpException:
                    self.escape_mode = False
                    raise
            
            if cmd == 3:
                raise executive.EndGameException
