# Copyright 2004 Tom Rothamel <tom-renpy@onegeek.org>
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#!/usr/bin/python

import re
import tpg

from fragment import *
import grammars
import image

class RPYParser(tpg.Parser):
    __doc__ = grammars.language_grammar

class IndentationParser(tpg.Parser):
    __doc__ = grammars.indent_grammar

    def __init__(self):
        super(IndentationParser, self).__init__()
        self.textlist = []
        self.indents = [ 0 ]

    def indent_dedent(self, s):

        ls = len(s)

        if ls > self.indents[-1]:
            self.accum("\020")
            self.indents.append(ls)
            return

        while True:
            if self.indents[-1] == ls:
                return 

            self.indents.pop()
            self.accum("\021")

            if len(self.indents) == 0:
                raise Exception, "Indent match error on line %d." % (
                    self.lexer.input[:self.lexer.pos].count('\n') + 1 )

    def accum(self, text):
        self.textlist.append(text)

    def text(self):
        return "".join(self.textlist) + "\021" * ( len(self.indents) - 1 )


def parse(f):

    fc = f.read()

    if fc.find("__end__") != 0:
        fc = fc[:fc.find("__end__")]
            
    ip = IndentationParser()
    ip(fc)

    rp = RPYParser()
    db, b = rp(ip.text())

    return db, b

