# Copyright 2004 Tom Rothamel <tom-renpy@onegeek.org>
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from params import config
import display

class Anchor(display.Showable):

    def __init__(self, showable, halign="center", valign="bottom",
                 xo=0, yo=0):
        self.laid_out = False
        self.showable = showable
        self.valign = valign
        self.halign = halign
        self.xo = xo
        self.yo = yo

    def layout(self):
        if self.laid_out:
            return       

        valign = self.valign
        halign = self.halign

        width, height = self.showable.size()
        swidth = config.screen_width
        sheight = config.screen_height

        if halign == "left":
            self.x = 0
        elif halign == "center":
            self.x = (swidth / 2) - (width / 2)
        elif halign == "right":
            self.x = swidth - width        

        if valign == "top":
            self.y = 0
        elif valign == "center":
            self.y = (sheight / 2) - (height / 2)
        elif valign == "bottom":
            self.y = sheight - height


        self.x += self.xo
        self.y += self.yo
        self.laid_out = True

    def draw(self, trans):
        self.layout()
        self.showable.draw(trans.move(self.x, self.y))

    def key_event(self, e):
        return self.showable.key_event(e)

    def mouse_event(self, button, x, y):
        return self.showable.mouse_event(button, x - self.x, y - self.y)

class VBox(display.Showable):

    def __init__(self, *args):
        self.contents = args

    def size(self):

        height = 0
        width = 0

        for i in self.contents:
            w, h = i.size()
            height += h + config.vbox_margin
            width = max(w, width)

        height -= config.vbox_margin

        
        return width, height

    def draw(self, trans):
        y = 0

        for i in self.contents:
            i.draw(trans.move(0, y))
            w, h = i.size()            
            y += h + config.vbox_margin

    def key_event(self, e):

        for i in self.contents:
            rv = i.key_event(e)

            if rv is not None:
                return rv

    def mouse_event(self, button, x, y):

        for i in self.contents:

            rv = i.mouse_event(button, x, y)

            if rv is not None:
                return rv

            w, h = i.size()
            y -= h + config.vbox_margin
            
            
