# Copyright 2004 Tom Rothamel <tom-renpy@onegeek.org>
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import optparse

import executive
import language
import gui
import textui
import dist
import sys

from params import config

def main(default_script, default_config):

    op = optparse.OptionParser()

    op.add_option("--script", dest="script", action="store",
                  default=default_script)

    op.add_option("--config", dest="config", action="store",
                  default=default_config)

    op.add_option("--dist", dest="dist", action="store",
                  default=None)

    options, args = op.parse_args()

    # Source in the config.
    execfile(options.config, config.__dict__)

    # Parse the script.
    f = file(options.script)
    declarations, block = language.parse(f)
    f.close()
        
    ui = gui.GUI()
    ui.run(declarations)

    if options.dist:
        dist.add(sys.argv[0])
        dist.add(options.script)
        dist.add(options.config)

        dist.dist(options.dist)

        sys.exit(0)

    try:
        ui.run(block)
    except executive.EndGameException:
        pass
    
